// Fang-Morris example
// BAB 11-3-2012
// Updated 11-6-2013

#include "fangmorris.hpp"
#include <vector>

void solveModel (double signalAccuracy);

int main(int argc, char ** argv)
{
  vector<double> accuracies (0);
  accuracies.push_back(0.8);

  for (double signalAccuracy: accuracies)
    solveModel(signalAccuracy);

  return 0;
}

void solveModel (double signalAccuracy) 
{
  int t, a, ahat;
  double entryCost=0.00;
  double minAngleIncrement = 0.05;
  int nbids=35;

  cout << "Signal strength = " << signalAccuracy << endl;

  cout << "Starting main" << endl;

  FangMorris fpa(nbids,entryCost,signalAccuracy);
  BCESolver solver(fpa);

  solver.setParameter(BCESolver::MinAngleIncrement,minAngleIncrement);

  cout << "Constructor finished" << endl;
  try
    {
      solver.populate();
      cout << "Done populating" << endl;

      char filename[100];
      sprintf(filename,"fangmorris_nb=%d_sig=%1.2f.bce",nbids,signalAccuracy);
      // sprintf(filename,"fangmorris_-p1+p0_nb=%d_entrycost_%1.2f_signalacc_%1.2f.bce",
      // 	      nbids,entryCost,signalAccuracy);

      IloCplex cplex=solver.getCplex();
      cplex.getObjective().setExpr(-solver.getObjectiveFunction(1)
				   +solver.getObjectiveFunction(0));
      cout << "Objective function set" << endl;
      
      // solver.solve();
      // cout << "Solved" << endl;

      // cout << "Objective = " << setprecision(16) << cplex.getObjValue() << endl;

      solver.mapBoundary();

      BCEData data;
      solver.getData(data);
      data.setNumValues(vector<int>(2,2));

      BCEData::save(data,filename);

      vector< vector<int> > actionConditions(2,vector<int>(0));
      vector< vector<int> > typeConditions(2,vector<int>(0));
      vector<int> stateConditions(0);

      // Calculate a conditional marginal distribution
      actionConditions[0] = vector<int>(3,0);
      actionConditions[0][1] = 1;
      actionConditions[0][2] = 2;
      stateConditions = vector<int>(1,3);

      vector<double> distr;

      data.getConditionalMarginal(stateConditions,actionConditions,typeConditions,
				  false, vector<bool>(2,false),vector<bool>(2,true),
				  distr);
    }
  catch (IloException & e)
    {
      cerr << "Concert exception caught: " << e << endl
	   << e.getMessage() << endl;
    }
  catch (BCEException & err)
    {
      cerr << "BCEException caught." << endl;
    }
  catch (...)
    {
      cout << "Unknown exception caught." << endl;
    }
}
