// Header file for first-price auction examples
// BAB 11-2012
// Updated 11-6-2013

#ifndef FPAKNOWN_HPP
#define FPAKNOWN_HPP

#include "bce.hpp"

class FPAKnown : public BCEGame
{
private:
  int numValues;
  double entryCost;
  double reservePrice; 

public:
  double highbid;
  distributionArray distribution;

  FPAKnown() {};
  
  FPAKnown(int na, int nv, double _entryCost, double _reservePrice) 
    : BCEGame(2,nv*nv,na,nv,6), numValues(nv), 
      entryCost(_entryCost), 
      reservePrice(_reservePrice),
      highbid(1.0)
  {
    unimodal uni(1.0,0.5,3);
    vToTheAlpha v2alph(1.0);
    independent stephensExample(1.0/3.0);
    // distribution.push_back(independent(1.0/3.0),1.0);
    distribution.push_back(vToTheAlpha(1.0),1.0);
  };

  int stateToPrivateValues(int state, vector<int> &values)
  {
    values.resize(2);

    values[0]=state%numValues;
    state-=values[0]; 
    values[1]=state/numValues;
  }

  double prior (int state, const vector<int> &types)
  {
    // Use a uniform prior
    vector<int> values(2,0);
    stateToPrivateValues(state,values);
    
    if (values[0]==types[0] && values[1]==types[1])
      {
	// if (values[0] != values[1])
	//   return 1.0/(numValues*(numValues-1.0));
	// return 0.0;

	// double p = 1.0/9.0;
	// double prob = 1.0;
	// if (values[0] == 0)
	//   prob *= p;
	// else
	//   prob *= 1-p;
	// if (values[1] == 0)
	//   prob *= p;
	// else
	//   prob *= 1-p;
	
	// return prob;
	
	double v0 = static_cast<double>(values[0]+1)/numValues;
	double v1 = static_cast<double>(values[1]+1)/numValues;
	double incr = 1.0/numValues;

	// return PDF(distribution,v0,v1,incr);
	return PDF(distribution,v0,v1,incr);
      }
    else
      return 0.0;
  }

  double objective(int state, const vector<int> &actions, int obj)
  {
    // Convert the state into a pair of valuations
    vector<int> values(2,0);
    stateToPrivateValues(state,values);
    double winningBid;

    if (obj < 2)
      {
	// Players 1 and 2
	double bid = highbid * actions[obj] / (numActions[obj]-1.0); // -1e-6;
	if (bid < reservePrice)
	  return 0.0;
	if (actions[obj] > actions[1-obj]
	    || (actions[0]==actions[1] && values[obj]>values[1-obj]))
	  return (1.0 * values[obj])/(numValues - 1.0) - bid
	    - (actions[obj]? entryCost : 0.0);
	if (actions[0] == actions[1] && values[0] == values[1])
	  return ((1.0 * values[obj])/(numValues - 1.0) - bid)/2.0
	    - (actions[obj]? entryCost : 0.0);
	
	return - (actions[obj]? entryCost : 0.0);
      }
    else if (obj==2)
      {
	// Revenue
	winningBid 
	  = (actions[0]>actions[1]? 
	     (highbid*actions[0])/(numActions[0]-1.0) : 
	     (highbid*actions[1])/(numActions[1]-1.0));
	return (winningBid >= reservePrice? winningBid : 0.0)
	  + (actions[0] > 0? entryCost : 0.0)
	  + (actions[1] > 0? entryCost : 0.0);
      }
    else if (obj==3)
      {
	// Surplus
	winningBid 
	  = (actions[0]>actions[1]? 
	     (highbid*actions[0])/(numActions[0]-1.0) : 
	     (highbid*actions[1])/(numActions[1]-1.0));
	if (winningBid < reservePrice)
	  return 0.0;
	if (actions[0]>actions[1]
	    || (actions[0]==actions[1] && values[0]>values[1]))
	  return (1.0*values[0]/(numValues-1.0)); // Player 1 won
	else
	  return (1.0*values[1]/(numValues-1.0)); // Either player 2 won or they tied with same val.
      }
    else if (obj == 4 || obj==5)
      {
	obj -= 4;
	// Players 1 and 2
	double bid = highbid * actions[obj] / (numActions[obj]-1.0); // -1e-6;
	if (actions[obj]>10)
	  {
	    if (bid < reservePrice)
	      return 0.0;
	    if (actions[obj] > actions[1-obj]
		|| (actions[0]==actions[1] && values[obj]>values[1-obj]))
	      return (1.0 * values[obj])/(numValues - 1.0) - bid
		- (actions[obj]? entryCost : 0.0);
	    if (actions[0] == actions[1] && values[0] == values[1])
	      return ((1.0 * values[obj])/(numValues - 1.0) - bid)/2.0
		- (actions[obj]? entryCost : 0.0);
	
	    return - (actions[obj]? entryCost : 0.0);
	  }
	return 0;
      }
  }

  bool dominated(int a, int t, int player)
  {
    if (((double) a)/(numActions[player] - 1.0) 
    	> ((double) t)/(numTypes[player] - 1.0))
      return (true);
    else if (((double) a)/(numActions[player]-1.0) < reservePrice-1e-10
	     && a)
      return true;
    else
      return (false);
    // return (false);
  }
};

#endif
